<?php

namespace Drupal\neverbounce\Controller;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides responses for NeverBounce routes.
 */
class NeverbounceController extends ControllerBase {

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Constructs a NeverbounceController object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ClientInterface $http_client) {
    $this->configFactory = $config_factory;
    $this->httpClient = $http_client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('http_client')
    );
  }

  /**
   * Validates an email using the NeverBounce API.
   *
   * @param string $email
   *   The email to validate.
   *
   * @return array
   *   The validation result.
   */
  public function validateEmail($email) {
    $config = $this->configFactory->get('neverbounce.settings');
    $api_key = $config->get('api_key');

    if (empty($api_key)) {
      return [
        '#markup' => $this->t('API key is missing. Please configure it in the settings.'),
        '#cache' => ['max-age' => 0],
      ];
    }

    try {
      $response = $this->httpClient->get('https://api.neverbounce.com/v4/single/check', [
        'query' => [
          'email' => $email,
          'key' => $api_key,
        ],
      ]);

      $data = json_decode($response->getBody(), TRUE);
      if (isset($data['result'])) {
        switch ($data['result']) {
          case 'valid':
            return ['#markup' => $this->t('The email is valid.')];
          case 'invalid':
            return ['#markup' => $this->t('The email is invalid.')];
          default:
            $message = $data['message'] ?? 'Unexpected result from NeverBounce.';
            return ['#markup' => $this->t($message)];
        }
      }
    } catch (RequestException $e) {
      $this->getLogger('neverbounce')->error($e->getMessage());
      return ['#markup' => $this->t('There was an error contacting the NeverBounce API.')];
    }

    // Add cacheable metadata to prevent caching.
    $cache_metadata = new CacheableMetadata();
    $cache_metadata->setCacheMaxAge(0);
    $cache_metadata->applyTo($output);

    return $output;
  }

  /**
   * Validates an email address via AJAX request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response containing the validation message.
   */
  public function validateEmailAjax(Request $request) {
    $email = $request->get('email');
    $config = $this->configFactory->get('neverbounce.settings');
    $api_key = $config->get('api_key');

    if (empty($api_key)) {
      return new JsonResponse(['message' => $this->t('API key is missing. Please configure it in the settings.')], 400);
    }

    if (filter_var($email, FILTER_VALIDATE_EMAIL) === FALSE) {
      return new JsonResponse(['message' => $this->t('Invalid email format.')], 400);
    }

    try {
      $response = $this->httpClient->get('https://api.neverbounce.com/v4/single/check', [
        'query' => [
          'email' => $email,
          'key' => $api_key,
        ],
      ]);

      $data = json_decode($response->getBody(), TRUE);

      if (isset($data['result'])) {
        switch ($data['result']) {
          case 'valid':
            return new JsonResponse(['message' => $this->t('The email is valid.')]);
          case 'invalid':
            return new JsonResponse(['message' => $this->t('The email is invalid.')]);
          default:
            $message = $data['message'] ?? 'Unexpected result from NeverBounce.';
            return new JsonResponse(['message' => $this->t($message)], 400);
        }
      }
    } catch (RequestException $e) {
      $this->getLogger('neverbounce')->error($e->getMessage());
      return new JsonResponse(['message' => $this->t('There was an error contacting the NeverBounce API.')], 500);
    }

    return new JsonResponse(['message' => $this->t('Unexpected error.')], 500);
  }

}
